@@ -403,7 +403,9 @@ unwanted_extensions = OptionList("misc", "unwanted_extensions", validation=lower
 action_on_unwanted_extensions = OptionNumber("misc", "action_on_unwanted_extensions", 0)
 unwanted_extensions_mode = OptionNumber("misc", "unwanted_extensions_mode", 0)
 new_nzb_on_failure = OptionBool("misc", "new_nzb_on_failure", False)
-history_retention = OptionStr("misc", "history_retention", "0")
+history_retention = OptionStr("misc", "history_retention", "0")  # Kept for converting to split option
+history_retention_option = OptionStr("misc", "history_retention_option", "all")
+history_retention_number = OptionNumber("misc", "history_retention_number", minval=1)
 
 quota_size = OptionStr("misc", "quota_size")
 quota_day = OptionStr("misc", "quota_day")
@@ -448,12 +450,13 @@ enable_filejoin = OptionBool("misc", "enable_filejoin", True)
 enable_tsjoin = OptionBool("misc", "enable_tsjoin", True)
 overwrite_files = OptionBool("misc", "overwrite_files", False)
 ignore_unrar_dates = OptionBool("misc", "ignore_unrar_dates", False)
-backup_for_duplicates = OptionBool("misc", "backup_for_duplicates", True)
+backup_for_duplicates = OptionBool("misc", "backup_for_duplicates", False)
 empty_postproc = OptionBool("misc", "empty_postproc", False)
 wait_for_dfolder = OptionBool("misc", "wait_for_dfolder", False)
 rss_filenames = OptionBool("misc", "rss_filenames", False)
 api_logging = OptionBool("misc", "api_logging", True)
 html_login = OptionBool("misc", "html_login", True)
+disable_archive = OptionBool("misc", "disable_archive", False)
 warn_dupl_jobs = OptionBool("misc", "warn_dupl_jobs", False)
 
 keep_awake = OptionBool("misc", "keep_awake", True)
@@ -461,6 +464,7 @@ tray_icon = OptionBool("misc", "tray_icon", True)
 allow_incomplete_nzb = OptionBool("misc", "allow_incomplete_nzb", False)
 enable_broadcast = OptionBool("misc", "enable_broadcast", True)
 ipv6_hosting = OptionBool("misc", "ipv6_hosting", False)
+ipv6_staging = OptionBool("misc", "ipv6_staging", False)
 api_warnings = OptionBool("misc", "api_warnings", True, protect=True)
 no_penalties = OptionBool("misc", "no_penalties", False)
 x_frame_options = OptionBool("misc", "x_frame_options", True)
@@ -612,6 +616,35 @@ pushbullet_prio_error = OptionBool("pushbullet", "pushbullet_prio_error", False)
 pushbullet_prio_queue_done = OptionBool("pushbullet", "pushbullet_prio_queue_done", False)
 pushbullet_prio_other = OptionBool("pushbullet", "pushbullet_prio_other", True)
 
+# [apprise]
+apprise_enable = OptionBool("apprise", "apprise_enable")
+apprise_cats = OptionList("apprise", "apprise_cats", ["*"])
+apprise_urls = OptionStr("apprise", "apprise_urls")
+apprise_target_startup = OptionStr("apprise", "apprise_target_startup")
+apprise_target_startup_enable = OptionBool("apprise", "apprise_target_startup_enable", False)
+apprise_target_download = OptionStr("apprise", "apprise_target_download")
+apprise_target_download_enable = OptionBool("apprise", "apprise_target_download_enable", False)
+apprise_target_pause_resume = OptionStr("apprise", "apprise_target_pause_resume")
+apprise_target_pause_resume_enable = OptionBool("apprise", "apprise_target_pause_resume_enable", False)
+apprise_target_pp = OptionStr("apprise", "apprise_target_pp")
+apprise_target_pp_enable = OptionBool("apprise", "apprise_target_pp_enable", False)
+apprise_target_complete = OptionStr("apprise", "apprise_target_complete")
+apprise_target_complete_enable = OptionBool("apprise", "apprise_target_complete_enable", True)
+apprise_target_failed = OptionStr("apprise", "apprise_target_failed")
+apprise_target_failed_enable = OptionBool("apprise", "apprise_target_failed_enable", True)
+apprise_target_disk_full = OptionStr("apprise", "apprise_target_disk_full")
+apprise_target_disk_full_enable = OptionBool("apprise", "apprise_target_disk_full_enable", False)
+apprise_target_new_login = OptionStr("apprise", "apprise_target_new_login")
+apprise_target_new_login_enable = OptionBool("apprise", "apprise_target_new_login_enable", True)
+apprise_target_warning = OptionStr("apprise", "apprise_target_warning")
+apprise_target_warning_enable = OptionBool("apprise", "apprise_target_warning_enable", False)
+apprise_target_error = OptionStr("apprise", "apprise_target_error")
+apprise_target_error_enable = OptionBool("apprise", "apprise_target_error_enable", False)
+apprise_target_queue_done = OptionStr("apprise", "apprise_target_queue_done")
+apprise_target_query_done_enable = OptionBool("apprise", "apprise_target_queue_done_enable", False)
+apprise_target_other = OptionStr("apprise", "apprise_target_other")
+apprise_target_other_enable = OptionBool("apprise", "apprise_target_other_enable", True)
+
 # [nscript]
 nscript_enable = OptionBool("nscript", "nscript_enable")
 nscript_cats = OptionList("nscript", "nscript_cats", ["*"])
