@@ -19,14 +19,14 @@
 sabnzbd.downloader - download engine
 """
 
-import time
 import select
 import logging
-from math import ceil
 from threading import Thread, RLock, current_thread
 import socket
 import sys
 import ssl
+import time
+from datetime import date
 from typing import List, Dict, Optional, Union, Set
 
 import sabnzbd
@@ -80,7 +80,6 @@ class Server:
         "required",
         "optional",
         "retention",
-        "send_group",
         "username",
         "password",
         "busy_threads",
@@ -111,7 +110,6 @@ class Server:
         use_ssl,
         ssl_verify,
         ssl_ciphers,
-        send_group,
         username=None,
         password=None,
         required=False,
@@ -134,15 +132,6 @@ class Server:
         self.required: bool = required
         self.optional: bool = optional
         self.retention: int = retention
-        self.send_group: bool = send_group
-
-        # TODO: Remove after next release
-        if send_group:
-            helpful_warning(
-                "You have 'Send Group' enabled for %s. Could you let us know why? https://github.com/sabnzbd/sabnzbd/discussions/2715",
-                self.displayname,
-            )
-
         self.username: Optional[str] = username
         self.password: Optional[str] = password
 
@@ -331,7 +320,6 @@ class Downloader(Thread):
             required = srv.required()
             optional = srv.optional()
             retention = int(srv.retention() * 24 * 3600)  # days ==> seconds
-            send_group = srv.send_group()
             create = True
 
         if oldserver:
@@ -358,7 +346,6 @@ class Downloader(Thread):
                     ssl,
                     ssl_verify,
                     ssl_ciphers,
-                    send_group,
                     username,
                     password,
                     required,
@@ -713,7 +700,7 @@ class Downloader(Thread):
     def process_nw(self, nw: NewsWrapper):
         """Receive data from a NewsWrapper and handle the response"""
         try:
-            bytes_received, done = nw.recv_chunk()
+            bytes_received, end_of_line, article_done = nw.recv_chunk()
         except ssl.SSLWantReadError:
             return
         except (ConnectionError, ConnectionAbortedError):
@@ -730,7 +717,7 @@ class Downloader(Thread):
                 self.last_max_chunk_size = bytes_received
             # Update statistics only when we fetched a whole article
             # The side effect is that we don't count things like article-not-available messages
-            if done:
+            if article_done:
                 article.nzf.nzo.update_download_stats(sabnzbd.BPSMeter.bps, server.id, nw.data_position)
             # Check speedlimit
             if (
@@ -742,7 +729,13 @@ class Downloader(Thread):
                     time.sleep(0.01)
                     sabnzbd.BPSMeter.update()
 
-        if nw.status_code != 222 and not done:
+        # If we are not at the end of a line, more data will follow
+        if not end_of_line:
+            return
+
+        # Response code depends on request command:
+        # 220 = ARTICLE, 222 = BODY
+        if nw.status_code not in (220, 222) and not article_done:
             if not nw.connected or nw.status_code == 480:
                 if not self.__finish_connect_nw(nw):
                     return
@@ -751,17 +744,11 @@ class Downloader(Thread):
                     self.__request_article(nw)
 
             elif nw.status_code == 223:
-                done = True
+                article_done = True
                 logging.debug("Article <%s> is present", article.article)
 
-            elif nw.status_code == 211:
-                logging.debug("group command ok -> %s", nw.nntp_msg)
-                nw.group = nw.article.nzf.nzo.group
-                nw.reset_data_buffer()
-                self.__request_article(nw)
-
-            elif nw.status_code in (411, 423, 430):
-                done = True
+            elif nw.status_code in (411, 423, 430, 451):
+                article_done = True
                 logging.debug(
                     "Thread %s@%s: Article %s missing (error=%s)",
                     nw.thrdnum,
@@ -783,7 +770,22 @@ class Downloader(Thread):
                 nw.reset_data_buffer()
                 self.__request_article(nw)
 
-        if done:
+            else:
+                # Don't warn for (internal) server errors during downloading
+                if nw.status_code not in (400, 502, 503):
+                    logging.warning(
+                        T("%s@%s: Received unknown status code %s for article %s"),
+                        nw.thrdnum,
+                        nw.server.host,
+                        nw.status_code,
+                        article.article,
+                    )
+
+                # Ditch this thread, we don't know what data we got now so the buffer can be bad
+                self.__reset_nw(nw, f"Server error or unknown status code: {nw.status_code}", wait=False)
+                return
+
+        if article_done:
             # Successful data, clear "bad" counter
             server.bad_cons = 0
             server.errormsg = server.warning = ""
@@ -975,16 +977,9 @@ class Downloader(Thread):
 
     def __request_article(self, nw: NewsWrapper):
         try:
-            nzo = nw.article.nzf.nzo
-            if nw.server.send_group and nzo.group != nw.group:
-                group = nzo.group
-                if sabnzbd.LOG_ALL:
-                    logging.debug("Thread %s@%s: GROUP <%s>", nw.thrdnum, nw.server.host, group)
-                nw.send_group(group)
-            else:
-                if sabnzbd.LOG_ALL:
-                    logging.debug("Thread %s@%s: BODY %s", nw.thrdnum, nw.server.host, nw.article.article)
-                nw.body()
+            if sabnzbd.LOG_ALL:
+                logging.debug("Thread %s@%s: BODY %s", nw.thrdnum, nw.server.host, nw.article.article)
+            nw.body()
             # Mark as ready to be read
             self.add_socket(nw.nntp.fileno, nw)
         except socket.error as err:
@@ -1125,9 +1120,12 @@ def check_server_expiration():
     """Check if user should get warning about server date expiration"""
     for server in config.get_servers().values():
         if server.expire_date():
-            days_to_expire = ceil(
-                (time.mktime(time.strptime(server.expire_date(), "%Y-%m-%d")) - time.time()) / (60 * 60 * 24)
-            )
+            try:
+                days_to_expire = (date.fromisoformat(server.expire_date()) - date.today()).days
+            except ValueError:
+                # In case of invalid date, just warn
+                days_to_expire = 0
+
             # Notify from 5 days in advance
             if days_to_expire < 6:
                 logging.warning(T("Server %s is expiring in %s day(s)"), server.displayname(), days_to_expire)
